@@ -2,42 +2,45 @@
 import warnings
 from contextlib import asynccontextmanager
 from logging import getLogger
+from typing import TYPE_CHECKING
 
 from aiohttp import ClientSession
 from aiohttp.typedefs import StrOrURL
 
-from aiohttp_client_cache.backends import CacheBackend
+from aiohttp_client_cache.backends import CacheBackend, get_valid_kwargs
 from aiohttp_client_cache.docs import copy_signature, extend_signature
-from aiohttp_client_cache.expiration import ExpirationTime
 from aiohttp_client_cache.response import AnyResponse, set_response_defaults
 
+if TYPE_CHECKING:
+    MIXIN_BASE = ClientSession
+else:
+    MIXIN_BASE = object
+
 logger = getLogger(__name__)
 
 
-class CacheMixin:
+class CacheMixin(MIXIN_BASE):
     """A mixin class for :py:class:`aiohttp.ClientSession` that adds caching support"""
 
     @extend_signature(ClientSession.__init__)
     def __init__(self, *, cache: CacheBackend = None, **kwargs):
-        super().__init__(**kwargs)  # type: ignore
         self.cache = cache or CacheBackend()
 
-    @copy_signature(ClientSession._request)
-    async def _request(
-        self, method: str, str_or_url: StrOrURL, expire_after: ExpirationTime = None, **kwargs
-    ) -> AnyResponse:
-        """Wrapper around :py:meth:`.SessionClient._request` that adds caching"""
-        cache_key = self.cache.create_key(method, str_or_url, **kwargs)
+        # Pass along any valid kwargs for ClientSession (or custom session superclass)
+        session_kwargs = get_valid_kwargs(super().__init__, kwargs)
+        super().__init__(**session_kwargs)
 
+    @copy_signature(ClientSession._request)
+    async def _request(self, method: str, str_or_url: StrOrURL, **kwargs) -> AnyResponse:
+        """Wrapper around :py:meth:`.SessionClient._request` that adds caching"""
         # Attempt to fetch cached response; if missing or expired, fetch new one
-        cached_response = await self.cache.get_response(cache_key)
-        if cached_response:
-            return cached_response
+        response, actions = await self.cache.request(method, str_or_url, **kwargs)
+        if response:
+            return response
         else:
-            logger.info(f'Cached response not found; making request to {str_or_url}')
+            logger.debug(f'Cached response not found; making request to {str_or_url}')
             new_response = await super()._request(method, str_or_url, **kwargs)  # type: ignore
-            await new_response.read()
-            await self.cache.save_response(cache_key, new_response, expire_after=expire_after)
+            await self.cache.save_response(new_response, actions)
             return set_response_defaults(new_response)
 
     @asynccontextmanager
